SystemOrganization addCategory: #'Khami-Core'!
SystemOrganization addCategory: #'Khami-Core-Announcements'!
SystemOrganization addCategory: #'Khami-Core-Support'!

Object subclass: #KmGame
	instanceVariableNames: 'tilePool players board announcer currentTile roads farms villages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmGame class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/9/2011 18:40'!
testGame
	^self basicNew
		initializeWithTilePool: KmTile testSet;
		yourself! !

!KmGame methodsFor: 'accessing' stamp: 'StuartHerring 1/12/2011 19:33'!
board
	^board! !

!KmGame methodsFor: 'accessing' stamp: 'StuartHerring 1/16/2011 20:19'!
currentTile
	^currentTile! !

!KmGame methodsFor: 'initialization' stamp: 'StuartHerring 1/30/2011 11:44'!
initializeWithTilePool: aPool
	self initialize.
	tilePool := aPool.
	board := KmGameBoard new.
	announcer := Announcer new.
	currentTile := nil.
	roads := OrderedCollection new.
	villages := OrderedCollection new.
	farms := OrderedCollection new.! !

!KmGame methodsFor: 'accessing' stamp: 'StuartHerring 1/30/2011 12:08'!
placeTileAt: row at: column
	(currentTile notNil and: [(board at: row at: column) isNil]) 
		ifTrue: [ board at: row at: column put: currentTile.
			currentTile := nil]! !

!KmGame methodsFor: 'accessing' stamp: 'StuartHerring 1/9/2011 18:26'!
remainingTiles
	^tilePool! !

!KmGame methodsFor: 'actions' stamp: 'StuartHerring 1/21/2011 13:27'!
takeTile
	^currentTile ifNil: [ currentTile := tilePool removeLast].! !

!KmGame methodsFor: 'actions' stamp: 'StuartHerring 1/30/2011 22:55'!
territories
	| result |
	result := Set new.
	board do: [ :tile | result addAll: tile territories].
	^result! !

Object subclass: #KmGameBoard
	instanceVariableNames: 'board startingTile'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmGameBoard methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 22:39'!
at: row at: column
	(row < 1 or: [column < 1 or: [row > board rows or: [column > board columns]]]) ifTrue: [^nil].
	^board at: row at: column! !

!KmGameBoard methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 22:34'!
at: row at: column put: aTile
	board at: row at: column put: aTile.
	self connectTilesAt: row at: column.
	(row = 1) ifTrue: [ board prependRow].
	(row = board rows) ifTrue: [board addRow].
	(column = 1) ifTrue: [ board prependColumn ].
	(column = board columns) ifTrue: [ board addColumn].! !

!KmGameBoard methodsFor: 'testing' stamp: 'StuartHerring 1/18/2011 19:17'!
canPlaceTile: aTile at: row at: column
	(board at: row at: column) ifNotNil: [^false].
	((board adjacentItemsAt: row at: column) noneSatisfy: [ :item | item notNil])
		ifTrue: [ ^false ].
	^KmLocation edges 
		allSatisfy: [ :side | self canPlaceTile: aTile toThe: side of: row and: column]
! !

!KmGameBoard methodsFor: 'testing' stamp: 'StuartHerring 1/18/2011 19:27'!
canPlaceTile: aTile toThe: aSide of: row and: column
	| r c |
	(aSide = Left) ifTrue: [ c := column - 1 ] 
		ifFalse: [ (aSide = Right) ifTrue: [ c := column + 1] ifFalse: [ c := column]].
	(aSide = Top) ifTrue: [ r := row - 1 ] 
		ifFalse: [ (aSide = Bottom) ifTrue: [ r := row + 1] ifFalse: [ r := row]].
	((r < 1) or: [c < 1]) ifTrue: [^true].
	((r > board rows) or: [c > board columns]) ifTrue: [^true].
	(board at: r at: c) ifNil: [^true].
	^(board at: r at: c) allowsTile: aTile at: aSide opposite! !

!KmGameBoard methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/12/2011 19:38'!
columns
	^board columns! !

!KmGameBoard methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 22:43'!
connectTilesAt: row at: column
	| tile |
	tile := board at: row at: column.
	tile tileAt: Top put: (self at: row - 1at: column).
	tile tileAt: Bottom put: (self at: row + 1at: column).
	tile tileAt: Left put: (self at: row at: column - 1).
	tile tileAt: Right put: (self at: row at: column + 1).
	! !

!KmGameBoard methodsFor: 'testing' stamp: 'StuartHerring 1/30/2011 23:05'!
do: aBlock
	^board do: [ :tile | tile ifNotNil: [ aBlock value: tile]]! !

!KmGameBoard methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/12/2011 19:49'!
initialize
	super initialize.
	startingTile := KmTile startingTile.
	board := KmGrid ofSize: 3 by: 3.
	board at: 2 at: 2 put: startingTile! !

!KmGameBoard methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/12/2011 19:38'!
rows
	^board rows! !

!KmGameBoard methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/12/2011 19:46'!
startingTile
	^startingTile! !

Object subclass: #KmGrid
	instanceVariableNames: 'rows cols'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core-Support'!

!KmGrid class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/10/2011 14:30'!
ofSize: rows by: columns
	^self new
		addColumns: columns;
		addRows: rows;
		yourself.! !

!KmGrid methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/17/2011 11:18'!
addColumn
	self addColumns: 1! !

!KmGrid methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/10/2011 14:28'!
addColumns: aNumber
	rows do:  [:row |
		aNumber timesRepeat: [ row add: nil ]].
	cols := cols + aNumber.
		! !

!KmGrid methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/17/2011 11:20'!
addRow
	self addRows: 1! !

!KmGrid methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/10/2011 14:07'!
addRows: aNumber
	aNumber timesRepeat:  [
		rows add: (OrderedCollection ofSize: cols)]! !

!KmGrid methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/18/2011 19:22'!
adjacentItemsAt: row at: column
	| return |
	return := OrderedCollection new.
	(row > 1) ifTrue: [ return add: (self at: row -1 at: column)].
	(column < cols) ifTrue: [ return add: (self at: row at: column + 1)].
	(row < rows size) ifTrue: [ return add: (self at: row  +1 at: column)].
	(column > 1) ifTrue: [ return add: (self at: row at: column - 1)].
	^return! !

!KmGrid methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/12/2011 09:44'!
at: row at: column
	^(rows at: row) at: column ! !

!KmGrid methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/12/2011 11:11'!
at: row at: column put: data
	(rows at: row) at: column put: data! !

!KmGrid methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/10/2011 11:59'!
columns
	^cols! !

!KmGrid methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 23:03'!
do: aBlock
	^rows do: [ :r | r do: [ :c | aBlock value: c]]! !

!KmGrid methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/10/2011 12:12'!
initialize
	super initialize.
	rows := OrderedCollection new.
	cols := 0.! !

!KmGrid methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/17/2011 11:50'!
prependColumn
	rows do:  [:row |
		 row addFirst: nil ].
	cols := cols + 1.
		! !

!KmGrid methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/17/2011 11:48'!
prependRow
	rows addFirst: (OrderedCollection ofSize: cols)! !

!KmGrid methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/10/2011 11:59'!
rows
	^rows size! !

Object subclass: #KmLocation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

KmLocation subclass: #BottomLeft
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!BottomLeft class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 15:54'!
connectingLocations
	^{LeftBottom. Bottom}! !

!BottomLeft class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 21:24'!
edge
	^Bottom! !

!BottomLeft class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/17/2011 18:18'!
opposite
	^TopLeft! !

KmLocation subclass: #BottomRight
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!BottomRight class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 15:54'!
connectingLocations
	^{Bottom. RightBottom}! !

!BottomRight class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 21:36'!
edge
	^Bottom! !

!BottomRight class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/17/2011 18:19'!
opposite
	^TopRight! !

KmLocation subclass: #Centre
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!Centre class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 15:55'!
connectingLocations
	^{Top. Right. Bottom. Left}! !

!Centre class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 21:37'!
edge
	self shouldNotImplement! !

!Centre class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/12/2011 13:21'!
opposite
	self shouldNotImplement! !

KmLocation subclass: #KmEdge
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

KmEdge subclass: #Bottom
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!Bottom class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 15:56'!
connectingLocations
	^{BottomLeft. Centre. BottomRight}! !

!Bottom class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 16:01'!
locations
	^{BottomRight. Bottom. BottomLeft}! !

!Bottom class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/12/2011 13:18'!
opposite
	^Top! !

!KmEdge class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 21:24'!
edge
	^self! !

!KmEdge class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 15:41'!
locations
	"The locations on this edge"
	self subclassResponsibility.! !

KmEdge subclass: #Left
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!Left class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 15:57'!
connectingLocations
	^{LeftBottom. Centre. LeftTop}. ! !

!Left class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 16:01'!
locations
	^{LeftTop. Left. LeftBottom.}! !

!Left class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/12/2011 13:19'!
opposite 
	^Right! !

KmEdge subclass: #Right
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!Right class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 15:57'!
connectingLocations
	^{RightBottom. Centre. RightTop}! !

!Right class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 16:02'!
locations
	^{RightTop. Right. RightBottom.}! !

!Right class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/12/2011 13:19'!
opposite
	^Left! !

KmEdge subclass: #Top
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!Top class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 15:52'!
connectingLocations
	^{TopLeft. Centre. TopRight}! !

!Top class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 16:02'!
locations
	^{TopLeft. Top. TopRight}! !

!Top class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/12/2011 13:19'!
opposite
	^Bottom! !

!KmLocation class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/28/2011 20:40'!
all
	^{TopLeft. Top. TopRight. 
		RightTop.  Right. RightBottom. 
		BottomRight. Bottom. BottomLeft. 
		LeftBottom. Left. LeftTop. 
		Centre}! !

!KmLocation class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 15:53'!
connectingLocations
	"The locations directly reachable from this location, starting from the right"
	self subclassResponsibility! !

!KmLocation class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 21:25'!
edge
	self subclassResponsibility.! !

!KmLocation class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/12/2011 13:22'!
edges
	^{Top. Right. Bottom. Left.}! !

!KmLocation class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/12/2011 13:17'!
opposite
	^self subclassResponsibility! !

!KmLocation class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/12/2011 13:22'!
pathLocations
	^{Top. Right. Bottom. Left. Centre}! !

KmLocation subclass: #LeftBottom
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!LeftBottom class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 15:58'!
connectingLocations
	^{Left. BottomLeft}! !

!LeftBottom class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 21:37'!
edge
	^Left! !

!LeftBottom class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/17/2011 18:19'!
opposite
	^RightBottom! !

KmLocation subclass: #LeftTop
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!LeftTop class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 15:58'!
connectingLocations
	^{TopLeft. Left}! !

!LeftTop class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 21:37'!
edge
	^Left! !

!LeftTop class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/17/2011 18:21'!
opposite
	^RightTop! !

KmLocation subclass: #RightBottom
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!RightBottom class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 15:58'!
connectingLocations
	^{BottomRight. Right}! !

!RightBottom class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 21:37'!
edge
	^Right! !

!RightBottom class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/17/2011 18:21'!
opposite
	^LeftBottom! !

KmLocation subclass: #RightTop
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!RightTop class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 15:59'!
connectingLocations
	^{Right. TopRight}! !

!RightTop class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 21:37'!
edge
	^Right! !

!RightTop class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/17/2011 18:21'!
opposite
	^LeftTop! !

KmLocation subclass: #TopLeft
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!TopLeft class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 15:59'!
connectingLocations
	^{Top. LeftTop}! !

!TopLeft class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 21:38'!
edge
	^Top! !

!TopLeft class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/17/2011 18:21'!
opposite
	^BottomLeft! !

KmLocation subclass: #TopRight
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!TopRight class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 15:59'!
connectingLocations
	^{RightTop. Top}! !

!TopRight class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 21:38'!
edge
	^Top! !

!TopRight class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/17/2011 18:21'!
opposite
	^BottomRight! !

Object subclass: #KmPiece
	instanceVariableNames: 'player'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmPiece class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/7/2011 15:07'!
forPlayer: aPlayer
	^self basicNew
		initializeWithPlayer: aPlayer;
		yourself! !

!KmPiece methodsFor: 'initialization' stamp: 'StuartHerring 1/7/2011 15:06'!
initializeWithPlayer: aPlayer
	self initialize.
	player := aPlayer! !

Object subclass: #KmPlayer
	instanceVariableNames: 'pieces colour name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

Object subclass: #KmTerritory
	instanceVariableNames: 'segments complete'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmTerritory methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 23:52'!
add: aSegment
	segments add: aSegment.
! !

!KmTerritory methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 21:49'!
initialize
	super initialize.
	segments := Set new.
	complete := false.! !

!KmTerritory methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/31/2011 21:23'!
isComplete
	^complete or: [ complete := segments noneSatisfy: [ :s | s isAtEdge]]! !

!KmTerritory methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 17:55'!
isEmpty
	^self size = 0.! !

!KmTerritory methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 23:53'!
mergeWith: aTerritory
	"Removes all segments from this territory and puts them in the provided one"
	aTerritory == self ifFalse: [
		segments do: [ :s | s territory: aTerritory].
		segments removeAll.
		aTerritory isComplete ifTrue: [
			aTerritory signalCompleteness]].! !

!KmTerritory methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/31/2011 21:21'!
signalCompleteness
	! !

!KmTerritory methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 16:08'!
size
	^segments size! !

!KmTerritory methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 19:37'!
tiles
	^Set withAll: (segments collect: [:s | s tile])! !

!KmTerritory methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 23:06'!
type
	^segments any type.! !

Object subclass: #KmTile
	instanceVariableNames: 'regions regionMap tiles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmTile class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 19:00'!
blank
	"Creates a blank tile consisting of nothing but farm"
	^self fromString: 'FFFFFFFFF'.! !

!KmTile class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/16/2011 17:07'!
buildTileSet: aCollection
	^(aCollection collect: [ :string | self fromString: string ]) asOrderedCollection! !

!KmTile class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 18:55'!
fromString: aString
	^self new
		buildFromString: aString;
		yourself! !

!KmTile class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/28/2011 21:08'!
startingTile
	^self fromString: 'VFFFV-F--'! !

!KmTile class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/31/2011 00:04'!
testSet
	^self buildTileSet: {
		'VVFFVVFFF'.
		'FFFF--FF-'.
		'FFFF--FF-'.
		'FFFF-FFFF'.
		'FFFFF-F--'.
		'VFFFVF---'.
		'FFFVF-FVV'.
		'VVVFVVVFF'.
		'VVVVVVVVV'.
		'VVVFVVVFF'.
		'VFFFV-F--'.
		'VFFFVFFFF'.
		'FFFFFFFFF'.
		'VVVFVVV-V'.
		'VFFFVFFFF'.
		'VVFFVVFFF'.
		'VFVFV-V--'.
		'VFFFV--F-'.
		'FFFFF-F--'.
		'FVFVFVFVV'.
		'FVFV-V-VV'.
	}! !

!KmTile methodsFor: 'initialization' stamp: 'StuartHerring 1/30/2011 19:02'!
allowsTile: aTile at: aSide
	(self tileAt: aSide) ifNotNil: [ ^false].
	aSide locations do: [ :r | ((self at: r) connectsTo: (aTile at: r opposite)) ifFalse: [ ^false]].
	^true! !

!KmTile methodsFor: 'accessing' stamp: 'StuartHerring 1/29/2011 13:04'!
alongPathAt: aSide do: aBlock
	"Follows a path outward and evaluates a block at every segment, passing in the current tile and region"
	self alongPathAt: aSide do: aBlock withHistory: Set new.! !

!KmTile methodsFor: 'accessing' stamp: 'StuartHerring 1/29/2011 13:07'!
alongPathAt: aSide do: aBlock withHistory: aSet
	"Follows a path outward and evaluates a block at every segment, passing in the current tile and region"
	| path nextTile |
	((aSet includes: (self at: aSide)) not and: [(self at: aSide) isPath]) ifTrue: [
		aSet add: (self at: aSide).
		aBlock value: self value: aSide.
		((nextTile := self tileAt: aSide) notNil 
			and: [(nextTile at: aSide opposite) connectsTo: (self at: aSide)])
				ifTrue: [ nextTile enterPathAt: aSide opposite do: aBlock withHistory: aSet]]! !

!KmTile methodsFor: 'accessing' stamp: 'StuartHerring 1/28/2011 20:31'!
at: aSymbol
	^regions at: (regionMap at: aSymbol)! !

!KmTile methodsFor: 'initialization' stamp: 'StuartHerring 1/30/2011 17:44'!
buildFromStream: aStream
	1 to: 4 do: [ :i | self buildSideFrom: aStream next at: i ].
	1 to: 5 do: [ :i | self buildPathFrom: aStream next at: i ].
	self calculateTerritories! !

!KmTile methodsFor: 'initialization' stamp: 'StuartHerring 1/30/2011 18:55'!
buildFromString: aString
	self buildFromStream: (ReadStream on: aString)! !

!KmTile methodsFor: 'initialization' stamp: 'StuartHerring 1/30/2011 17:46'!
buildPathFrom: char at: index
	self buildSegmentFrom: char at: (index = 5 ifTrue: [ 13 ] ifFalse: [ index * 3 - 1 ]).! !

!KmTile methodsFor: 'initialization' stamp: 'StuartHerring 1/30/2011 17:45'!
buildSegmentFrom: char at: index
	(regions at: index  put: (KmTileSegment fromChar: char in: self))
		location: (regionMap keyAtValue: index)! !

!KmTile methodsFor: 'initialization' stamp: 'StuartHerring 1/30/2011 17:46'!
buildSideFrom: char at: index
	self buildSegmentFrom: char at:  index * 3 - 2.
	self buildSegmentFrom: char at:  index * 3.! !

!KmTile methodsFor: 'initialization' stamp: 'StuartHerring 1/30/2011 21:13'!
calculateTerritories
	KmLocation all do: [ :r | | seg1 | seg1 := (self at: r).
		r connectingLocations do: [ :r2 | | seg2 | seg2 := (self at: r2).
			(seg1 connectsTo: seg2) 
				ifTrue: [	seg2 territory mergeWith: seg1 territory]]].! !

!KmTile methodsFor: 'accessing' stamp: 'StuartHerring 1/29/2011 12:51'!
collectPathAt: aSide
	"Collects continuous tiles along a path starting with the path at the designated side"
	| result |
	result := OrderedCollection new.
	self alongPathAt: aSide do: [ :tile : region | (result includes: tile) ifFalse: [result add: tile]].
	^result.! !

!KmTile methodsFor: 'accessing' stamp: 'StuartHerring 1/29/2011 13:10'!
enterPathAt: aSide do: aBlock withHistory: aSet
	"Follows a path inward from a side and evaluates a block at every segment, passing in the current tile and region"
	| path nextTile |
	((aSet includes: (self at: aSide)) not and: [(self at: aSide) isPath]) ifTrue: [
		aSet add: (self at: aSide).
		aBlock value: self value: aSide.
		((aSet includes: (self at: Centre)) not and: [((self at: Centre) connectsTo: (self at: aSide))]) ifTrue: [ | nextSide |
			aSet add:(self at: Centre).
			aBlock value: self value: Centre.
			((KmLocation edges reject: [ :s | s = aSide]) detect: [:s | (self at: s) connectsTo: (self at: aSide)] ifNone: [nil])
				ifNotNil: [ :s | self alongPathAt: s do: aBlock withHistory: aSet]]]! !

!KmTile methodsFor: 'initialization' stamp: 'StuartHerring 1/30/2011 18:59'!
initialize
	super initialize.
	tiles := Dictionary new.
	regions := Dictionary new.
	regionMap := Dictionary new.
	KmLocation all
		keysAndValuesDo: [ :index :region | regionMap at: region put: index ].
	territories := Set new.! !

!KmTile methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 22:23'!
rotate
	regionMap keysAndValuesDo: [ :key : value |
		value = 13 ifFalse: [ | new |
			new := ((value + 8 ) \\ 12) + 1.
			regionMap at: key put: new.
			(regions at: new) location: key]].! !

!KmTile methodsFor: 'initialization' stamp: 'StuartHerring 1/30/2011 21:14'!
territories
	^Set new 
		addAll: (regions select: [ :r | r territory notNil ] thenCollect: [ :r | r territory]);
		yourself! !

!KmTile methodsFor: 'accessing' stamp: 'StuartHerring 1/12/2011 13:10'!
tileAt: aDirection
	^tiles at: aDirection ifAbsent: [nil]! !

!KmTile methodsFor: 'accessing' stamp: 'StuartHerring 1/30/2011 22:43'!
tileAt: aDirection put: aTile
	aTile ifNil: [^nil].
	tiles at: aDirection ifAbsent: [
		tiles at: aDirection put: aTile.
		aTile tileAt: aDirection opposite put: self].
	aDirection locations do: [ :loc | (self at: loc) territory mergeWith: (aTile at: loc opposite) territory].
	^aTile! !

Object subclass: #KmTileSegment
	instanceVariableNames: 'territory tile location'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

KmTileSegment subclass: #KmBlocker
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmBlocker class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 17:57'!
codeCharacter
	^$.! !

!KmBlocker methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 18:23'!
territory
	^nil! !

!KmBlocker methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 18:24'!
territory: aTerritory
	"do nothing"! !

!KmBlocker methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 23:06'!
type
	^#blocker! !

KmTileSegment subclass: #KmPath
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmPath methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/29/2011 12:13'!
isPath
	^true! !

KmPath subclass: #KmRoadPath
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmRoadPath class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/8/2011 22:59'!
codeCharacter
	^$-! !

!KmRoadPath methodsFor: 'testing' stamp: 'StuartHerring 1/17/2011 17:50'!
connectsTo: aSegment
	^aSegment isRoad! !

!KmRoadPath methodsFor: 'testing' stamp: 'StuartHerring 1/8/2011 22:58'!
isRoad
	^true! !

!KmRoadPath methodsFor: 'testing' stamp: 'StuartHerring 1/30/2011 23:07'!
type
	^#road! !

KmTileSegment subclass: #KmRegion
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

KmRegion subclass: #KmFarmRegion
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmFarmRegion class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/8/2011 22:42'!
codeCharacter
	^$F! !

!KmFarmRegion methodsFor: 'testing' stamp: 'StuartHerring 1/17/2011 17:51'!
connectsTo: aSegment
	^aSegment isFarm! !

!KmFarmRegion methodsFor: 'testing' stamp: 'StuartHerring 1/8/2011 23:10'!
isFarm
	^true! !

!KmFarmRegion methodsFor: 'testing' stamp: 'StuartHerring 1/30/2011 23:07'!
type
	^#farm! !

KmRegion subclass: #KmVillageRegion
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Khami-Core'!

!KmVillageRegion class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/8/2011 22:42'!
codeCharacter
	^$V! !

!KmVillageRegion methodsFor: 'testing' stamp: 'StuartHerring 1/17/2011 17:51'!
connectsTo: aSegment
	^aSegment isVillage! !

!KmVillageRegion methodsFor: 'testing' stamp: 'StuartHerring 1/8/2011 23:11'!
isVillage
	^true! !

!KmVillageRegion methodsFor: 'testing' stamp: 'StuartHerring 1/30/2011 23:07'!
type
	^#village! !

!KmTileSegment class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/29/2011 12:44'!
codeCharacter
	^nil! !

!KmTileSegment class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 16:18'!
fromChar: aChar in: aTile.
	^(self allSubclasses detect: [ :class | 
		(class isAbstract not) and: [class codeCharacter = aChar]]) in: aTile.! !

!KmTileSegment class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 16:22'!
in: aTile
	^self basicNew
		initializeWithTile: aTile;
		yourself! !

!KmTileSegment class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/29/2011 12:46'!
isAbstract
	^self codeCharacter isNil! !

!KmTileSegment class methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 17:50'!
new
	self shouldNotImplement.! !

!KmTileSegment methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/17/2011 17:42'!
connectsTo: aSegment
	^false! !

!KmTileSegment methodsFor: 'testing' stamp: 'StuartHerring 1/31/2011 21:21'!
initializeWithTile: aTile
	self initialize.
	tile := aTile.
	self territory: KmTerritory new.! !

!KmTileSegment methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 22:04'!
isAtEdge
	^location = Centre 
		ifTrue: [false]
		ifFalse: [ location isNil or: [(tile tileAt: location edge) isNil]]! !

!KmTileSegment methodsFor: 'testing' stamp: 'StuartHerring 1/31/2011 21:21'!
isComplete
	^territory isComplete! !

!KmTileSegment methodsFor: 'testing' stamp: 'StuartHerring 1/8/2011 22:59'!
isFarm 
	^false! !

!KmTileSegment methodsFor: 'testing' stamp: 'StuartHerring 1/29/2011 12:12'!
isPath
	^false! !

!KmTileSegment methodsFor: 'testing' stamp: 'StuartHerring 1/9/2011 17:15'!
isRoad
	^false! !

!KmTileSegment methodsFor: 'testing' stamp: 'StuartHerring 1/30/2011 17:50'!
isVillage 
	^false! !

!KmTileSegment methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 16:16'!
location
	^location! !

!KmTileSegment methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 16:23'!
location: aLocation
	location := aLocation.! !

!KmTileSegment methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 18:20'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ', '.
	aStream nextPutAll: location name! !

!KmTileSegment methodsFor: 'testing' stamp: 'StuartHerring 1/30/2011 14:02'!
territory
	^territory! !

!KmTileSegment methodsFor: 'testing' stamp: 'StuartHerring 1/30/2011 17:48'!
territory: aTerritory
	territory := aTerritory.
	aTerritory add: self.! !

!KmTileSegment methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 16:16'!
tile
	^tile! !

!KmTileSegment methodsFor: 'as yet unclassified' stamp: 'StuartHerring 1/30/2011 23:06'!
type
	self subclassResponsibility ! !
